package org.opengroup.osdu.register.provider.azure.subscriber;

import com.microsoft.azure.arm.model.Indexable;
import com.microsoft.azure.management.eventgrid.v2019_01_01.WebHookEventSubscriptionDestination;
import com.microsoft.azure.management.eventgrid.v2019_01_01.implementation.EventGridManager;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.register.provider.azure.di.AzureBootstrapConfig;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.utils.AppServiceConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.atomic.AtomicBoolean;

@Component
public class PushSubscription {

    @Autowired
    private EventGridManager eventGridManager;

    @Autowired
    private AzureBootstrapConfig azureBootstrapConfig;

    @Autowired
    private JaxRsDpsLog logger;

    @Autowired
    private AppServiceConfig serviceConfig;

    public boolean createPushSubscription(Subscription subscription) {
        WebHookEventSubscriptionDestination subscriptionDestination = new WebHookEventSubscriptionDestination();
        subscriptionDestination.withEndpointUrl(serviceConfig.getRecordsChangePubsubEndpoint());

        String subscriptionId = subscription.getNotificationId();
        String scope = String.format("/subscriptions/%s/resourceGroups/%s/providers/Microsoft.EventGrid/topics/%s", azureBootstrapConfig.getAzureSubscriptionId(), azureBootstrapConfig.getResourceGroupName(), subscription.getTopic());
        AtomicBoolean res = new AtomicBoolean(true);

        eventGridManager.eventSubscriptions().define(subscriptionId)
                .withScope(scope)
                .withDestination(subscriptionDestination)
                .createAsync()
                .toBlocking()
                .subscribe(
                        (Indexable indexable) -> {},
                        (Throwable throwable) -> {
                                logger.error(String.format("Creating subscription with id %s failed with error %s", subscriptionId, throwable.getMessage()));
                                res.set(false);
                            },
                        () -> logger.info(String.format("Subscription with id %s created successfully", subscriptionId))
                );

        return res.get();

    }

    public boolean deletePushSubscription(String subscriptionId, String topicName) {
        AtomicBoolean res = new AtomicBoolean(true);
        String scope = String.format("/subscriptions/%s/resourceGroups/%s/providers/Microsoft.EventGrid/topics/%s", azureBootstrapConfig.getAzureSubscriptionId(), azureBootstrapConfig.getResourceGroupName(), topicName);

        eventGridManager.eventSubscriptions().deleteAsync(scope, subscriptionId)
                .subscribe(() -> {
                    logger.info(String.format("Subscription with id %s deleted successfully", subscriptionId));
                }, (Throwable error) -> {
                    logger.error(String.format("Delete operation on subscription with id %s failed with error %s", subscriptionId, error.getMessage()));
                    res.set(false);
                });

        return res.get();
    }

}
