package org.opengroup.osdu.register.provider.azure.di;

import com.azure.security.keyvault.secrets.SecretClient;
import com.azure.security.keyvault.secrets.models.KeyVaultSecret;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.doReturn;

@ExtendWith(MockitoExtension.class)
public class AzureBootstrapConfigTest {

    @InjectMocks
    private AzureBootstrapConfig bootstrapConfig;

    @Mock
    private KeyVaultSecret secret;

    @Mock
    private SecretClient kv;

    @Test
    public void kvSecretChecksForNullResponse() {

        doReturn(null).when(kv).getSecret("secret-name");

        IllegalStateException exception = assertThrows(IllegalStateException.class, () ->{
            bootstrapConfig.getKeyVaultSecret(kv, "secret-name");
        });

        assertEquals("No secret found with name secret-name", exception.getMessage());
    }

    @Test
    public void kvSecretChecksForNullValueWithinResponse() {
        doReturn(null).when(secret).getValue();
        doReturn(secret).when(kv).getSecret("secret-name");

        IllegalStateException exception = assertThrows(IllegalStateException.class, () ->{
            bootstrapConfig.getKeyVaultSecret(kv, "secret-name");
        });

        assertEquals("Secret unexpectedly missing from KeyVault response for secret with name secret-name", exception.getMessage());
    }

    @Test
    public void configReturnsCorrectSecretCosmosKey() {
        doReturn("cosmos-key-secret").when(secret).getValue();
        doReturn(secret).when(kv).getSecret("cosmos-db-key");

        String secretValue = bootstrapConfig.cosmosKey(kv);
        assertEquals( "cosmos-key-secret", secretValue);
    }

    @Test
    public void configReturnsCorrectSecretCosmosEndpoint() {
        doReturn("cosmos-endpoint-secret").when(secret).getValue();
        doReturn(secret).when(kv).getSecret("cosmosdb-uri");

        String secretValue = bootstrapConfig.cosmosEndpoint(kv);
        assertEquals( "cosmos-endpoint-secret", secretValue);
    }
}
